@@ -111,7 +111,7 @@ class BBCooker:
 
         return self.tryBuildPackage(fn, item, self.configuration.cmd, the_data, build_depends)
 
-    def showVersions( self ):
+    def showVersions(self):
         pkg_pn = self.status.pkg_pn
         preferred_versions = {}
         latest_versions = {}
@@ -130,11 +130,11 @@ class BBCooker:
             latest = latest_versions[p]
 
             if pref != latest:
-                prefstr = pref[0][0] + "-" + pref[0][1]
+                prefstr = pref[0][0] + ":" + pref[0][1] + '-' + pref[0][2]
             else:
                 prefstr = ""
 
-            print "%-30s %20s %20s" % (p, latest[0][0] + "-" + latest[0][1],
+            print "%-30s %20s %20s" % (p, latest[0][0] + ":" + latest[0][1] + "-" + latest[0][2],
                                         prefstr)
 
 
@@ -201,7 +201,7 @@ class BBCooker:
             fnid = rq.runq_fnid[task]
             fn = taskdata.fn_index[fnid]
             pn = self.status.pkg_fn[fn]
-            version  = self.bb_cache.getVar('PV', fn, True ) + '-' + self.bb_cache.getVar('PR', fn, True)
+            version  = "%s:%s-%s" % self.status.pkg_pepvpr[fn]
             print >> tdepends_file, '"%s.%s" [label="%s %s\\n%s\\n%s"]' % (pn, taskname, pn, taskname, version, fn)
             for dep in rq.runq_depends[task]:
                 depfn = taskdata.fn_index[rq.runq_fnid[dep]]
