@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # ex:ts=4:sw=4:sts=4:et
 # -*- tab-width: 4; c-basic-offset: 4; indent-tabs-mode: nil -*-
 """
@@ -18,26 +17,26 @@ to have faster update_data and expandKeys operations.
 
 This is a treade-off between speed and memory again but
 the speed is more critical here.
-
-Copyright (C) 2003, 2004  Chris Larson
-Copyright (C) 2005        Holger Hans Peter Freyther
-
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU General Public License as published by the Free Software
-Foundation; either version 2 of the License, or (at your option) any later
-version.
-
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License along with
-this program; if not, write to the Free Software Foundation, Inc., 59 Temple
-Place, Suite 330, Boston, MA 02111-1307 USA. 
-
-Based on functions from the base bb module, Copyright 2003 Holger Schurig
 """
 
+# Copyright (C) 2003, 2004  Chris Larson
+# Copyright (C) 2005        Holger Hans Peter Freyther
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as
+# published by the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+#Based on functions from the base bb module, Copyright 2003 Holger Schurig
+
 import sys, os, re, time, types
 if sys.argv[0][-5:] == "pydoc":
     path = os.path.dirname(os.path.dirname(sys.argv[1]))
@@ -271,6 +270,9 @@ def expandKeys(alterdata, readdata = None):
         readdata = alterdata
 
     for key in keys(alterdata):
+        if not '${' in key:
+            continue
+
         ekey = expand(key, readdata)
         if key == ekey:
             continue
@@ -363,11 +365,16 @@ def emit_var(var, o=sys.__stdout__, d = init(), all=False):
     val.rstrip()
     if not val:
         return 0
+	
+    varExpanded = expand(var, d)
 
     if getVarFlag(var, "func", d):
 #       NOTE: should probably check for unbalanced {} within the var
-        o.write("%s() {\n%s\n}\n" % (var, val))
+        o.write("%s() {\n%s\n}\n" % (varExpanded, val))
     else:
+        if getVarFlag(var, "unexport", d):
+            o.write('unset %s\n' % varExpanded)
+            return 1
         if getVarFlag(var, "export", d):
             o.write('export ')
         else:
@@ -376,7 +383,7 @@ def emit_var(var, o=sys.__stdout__, d = init(), all=False):
 #       if we're going to output this within doublequotes,
 #       to a shell, we need to escape the quotes in the var
         alter = re.sub('"', '\\"', val.strip())
-        o.write('%s="%s"\n' % (var, alter))
+        o.write('%s="%s"\n' % (varExpanded, alter))
     return 1
 
 
@@ -534,8 +541,8 @@ def update_data(d):
 
 
 def inherits_class(klass, d):
-    val = getVar('__inherit_cache', d) or ""
-    if os.path.join('classes', '%s.bbclass' % klass) in val.split():
+    val = getVar('__inherit_cache', d) or []
+    if os.path.join('classes', '%s.bbclass' % klass) in val:
         return True
     return False
 
