@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # ex:ts=4:sw=4:sts=4:et
 # -*- tab-width: 4; c-basic-offset: 4; indent-tabs-mode: nil -*-
 """
@@ -6,25 +5,25 @@ BitBake 'Fetch' implementations
 
 Classes for obtaining upstream sources for the
 BitBake build tools.
-
-Copyright (C) 2003, 2004  Chris Larson
-
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU General Public License as published by the Free Software
-Foundation; either version 2 of the License, or (at your option) any later
-version.
-
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License along with
-this program; if not, write to the Free Software Foundation, Inc., 59 Temple
-Place, Suite 330, Boston, MA 02111-1307 USA. 
-
-Based on functions from the base bb module, Copyright 2003 Holger Schurig
 """
 
+# Copyright (C) 2003, 2004  Chris Larson
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as
+# published by the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+# Based on functions from the base bb module, Copyright 2003 Holger Schurig
+
 import os, re
 import bb
 from   bb import data
@@ -75,6 +74,7 @@ def uri_replace(uri, uri_find, uri_replace, d):
     return bb.encodeurl(result_decoded)
 
 methods = []
+urldata = {}
 
 def init(urls = [], d = None):
     if d == None:
@@ -85,52 +85,88 @@ def init(urls = [], d = None):
         m.urls = []
 
     for u in urls:
+        ud = initdata(u, d)
+        if ud.method:
+            ud.method.urls.append(u)
+
+def initdata(url, d):
+    fn = bb.data.getVar('FILE', d, 1)
+    if fn not in urldata:
+        urldata[fn] = {}
+    if url not in urldata[fn]:
+        ud = FetchData()
+        (ud.type, ud.host, ud.path, ud.user, ud.pswd, ud.parm) = bb.decodeurl(data.expand(url, d))
+        ud.date = Fetch.getSRCDate(ud, d)
         for m in methods:
-            m.data = d
-            if m.supports(u, d):
-                m.urls.append(u)
+            if m.supports(url, ud, d):
+                ud.localpath = m.localpath(url, ud, d)
+                ud.md5 = ud.localpath + '.md5'
+                # if user sets localpath for file, use it instead.
+                if "localpath" in ud.parm:
+                    ud.localpath = ud.parm["localpath"]
+                ud.method = m
+                break
+        urldata[fn][url] = ud
+    return urldata[fn][url]
 
 def go(d):
     """Fetch all urls"""
+    fn = bb.data.getVar('FILE', d, 1)
     for m in methods:
-        if m.urls:
-            m.go(d)
+        for u in m.urls:
+            ud = urldata[fn][u]
+            if ud.localfile and not m.forcefetch(u, ud, d) and os.path.exists(urldata[fn][u].md5):
+                # File already present along with md5 stamp file
+                # Touch md5 file to show activity
+                os.utime(ud.md5, None)
+                continue
+            # RP - is olddir needed?
+            # olddir = os.path.abspath(os.getcwd())
+            m.go(u, ud	, d)
+            # os.chdir(olddir)
+            if ud.localfile and not m.forcefetch(u, ud, d):
+                Fetch.write_md5sum(u, ud, d)
 
 def localpaths(d):
     """Return a list of the local filenames, assuming successful fetch"""
     local = []
+    fn = bb.data.getVar('FILE', d, 1)
     for m in methods:
         for u in m.urls:
-            local.append(m.localpath(u, d))
+            local.append(urldata[fn][u].localpath)
     return local
 
 def localpath(url, d):
-    for m in methods:
-        if m.supports(url, d):
-            return m.localpath(url, d)
+    ud = initdata(url, d)
+    if ud.method:
+        return ud.localpath
     return url
 
+class FetchData(object):
+    """Class for fetcher variable store"""
+    def __init__(self):
+        self.localfile = ""
+
+
 class Fetch(object):
     """Base class for 'fetch'ing data"""
 
     def __init__(self, urls = []):
         self.urls = []
-        for url in urls:
-            if self.supports(bb.decodeurl(url), d) is 1:
-                self.urls.append(url)
 
-    def supports(url, d):
-        """Check to see if this fetch class supports a given url.
-           Expects supplied url in list form, as outputted by bb.decodeurl().
+    def supports(self, url, urldata, d):
+        """
+        Check to see if this fetch class supports a given url.
         """
         return 0
-    supports = staticmethod(supports)
 
-    def localpath(url, d):
-        """Return the local filename of a given url assuming a successful fetch.
+    def localpath(self, url, urldata, d):
+        """
+        Return the local filename of a given url assuming a successful fetch.
+        Can also setup variables in urldata for use in go (saving code duplication 
+        and duplicate code execution)
         """
         return url
-    localpath = staticmethod(localpath)
 
     def setUrls(self, urls):
         self.__urls = urls
@@ -140,25 +176,34 @@ class Fetch(object):
 
     urls = property(getUrls, setUrls, None, "Urls property")
 
-    def setData(self, data):
-        self.__data = data
+    def forcefetch(self, url, urldata, d):
+        """
+        Force a fetch, even if localpath exists?
+        """
+        return False
 
-    def getData(self):
-        return self.__data
-
-    data = property(getData, setData, None, "Data property")
-
-    def go(self, urls = []):
-        """Fetch urls"""
+    def go(self, url, urldata, d):
+        """
+        Fetch urls
+        Assumes localpath was called first
+        """
         raise NoMethodError("Missing implementation for url")
 
-    def getSRCDate(d):
+    def getSRCDate(urldata, d):
         """
         Return the SRC Date for the component
 
         d the bb.data module
         """
-        return data.getVar("SRCDATE", d, 1) or data.getVar("CVSDATE", d, 1) or data.getVar("DATE", d, 1 )
+        if "srcdate" in urldata.parm:
+            return urldata.parm['srcdate']
+
+        pn = data.getVar("PN", d, 1)
+
+        if pn:
+            return data.getVar("SRCDATE_%s" % pn, d, 1) or data.getVar("CVSDATE_%s" % pn, d, 1) or data.getVar("DATE", d, 1)
+
+        return data.getVar("SRCDATE", d, 1) or data.getVar("CVSDATE", d, 1) or data.getVar("DATE", d, 1)
     getSRCDate = staticmethod(getSRCDate)
 
     def try_mirror(d, tarfn):
@@ -193,27 +238,36 @@ class Fetch(object):
         return False
     try_mirror = staticmethod(try_mirror)
 
-    def check_for_tarball(d, tarfn, dldir, date):
+    def verify_md5sum(ud, got_sum):
         """
-        Check for a local copy then check the tarball stash.
-        Both checks are skipped if date == 'now'.
-
-        d Is a bb.data instance
-        tarfn is the name of the tarball
-        date is the SRCDATE
+        Verify the md5sum we wanted with the one we got
         """
-        if "now" != date:
-            dl = os.path.join(dldir, tarfn)
-            if os.access(dl, os.R_OK):
-                bb.msg.debug(1, bb.msg.domain.Fetcher, "%s already exists, skipping checkout." % tarfn)
-                return True
+        wanted_sum = None
+        if 'md5sum' in ud.parm:
+            wanted_sum = ud.parm['md5sum']
+        if not wanted_sum:
+            return True
 
-            # try to use the tarball stash
-            if Fetch.try_mirror(d, tarfn):
-                return True
-        return False
-    check_for_tarball = staticmethod(check_for_tarball)
+        return wanted_sum == got_sum
+    verify_md5sum = staticmethod(verify_md5sum)
 
+    def write_md5sum(url, ud, d):
+        if bb.which(data.getVar('PATH', d), 'md5sum'):
+            try:
+                md5pipe = os.popen('md5sum ' + ud.localpath)
+                md5data = (md5pipe.readline().split() or [ "" ])[0]
+                md5pipe.close()
+            except OSError:
+                md5data = ""
+
+        # verify the md5sum
+        if not Fetch.verify_md5sum(ud, md5data):
+            raise MD5SumError(url)
+
+        md5out = file(ud.md5, 'w')
+        md5out.write(md5data)
+        md5out.close()
+    write_md5sum = staticmethod(write_md5sum)
 
 import cvs
 import git
@@ -221,6 +275,8 @@ import local
 import svn
 import wget
 import svk
+import ssh
+import perforce
 
 methods.append(cvs.Cvs())
 methods.append(git.Git())
@@ -228,3 +284,5 @@ methods.append(local.Local())
 methods.append(svn.Svn())
 methods.append(wget.Wget())
 methods.append(svk.Svk())
+methods.append(ssh.SSH())
+methods.append(perforce.Perforce())
