@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # ex:ts=4:sw=4:sts=4:et
 # -*- tab-width: 4; c-basic-offset: 4; indent-tabs-mode: nil -*-
 ##########################################################################
@@ -6,17 +5,18 @@
 # Copyright (C) 2005-2006 Michael 'Mickey' Lauer <mickey@Vanille.de>
 # Copyright (C) 2005-2006 Vanille Media
 #
-# This program is free software; you can redistribute it and/or modify it under
-# the terms of the GNU General Public License as published by the Free Software
-# Foundation; version 2 of the License.
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as
+# published by the Free Software Foundation.
 #
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License along with
-# this program; if not, write to the Free Software Foundation, Inc., 59 Temple
-# Place, Suite 330, Boston, MA 02111-1307 USA.
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 ##########################################################################
 #
@@ -104,10 +104,11 @@ class BitBakeShellCommands:
 
     def _findProvider( self, item ):
         self._checkParsed()
+        # Need to use taskData for this information
         preferred = data.getVar( "PREFERRED_PROVIDER_%s" % item, cooker.configuration.data, 1 )
         if not preferred: preferred = item
         try:
-            lv, lf, pv, pf = Providers.findBestProvider(preferred, cooker.configuration.data, cooker.status, cooker.build_cache_fail)
+            lv, lf, pv, pf = Providers.findBestProvider(preferred, cooker.configuration.data, cooker.status)
         except KeyError:
             if item in cooker.status.providers:
                 pf = cooker.status.providers[item][0]
@@ -144,6 +145,7 @@ class BitBakeShellCommands:
 
     def build( self, params, cmd = "build" ):
         """Build a providee"""
+        global last_exception
         globexpr = params[0]
         self._checkParsed()
         names = globfilter( cooker.status.pkg_pn.keys(), globexpr )
@@ -152,10 +154,8 @@ class BitBakeShellCommands:
 
         oldcmd = cooker.configuration.cmd
         cooker.configuration.cmd = cmd
-        cooker.build_cache = []
-        cooker.build_cache_fail = []
 
-        td = taskdata.TaskData()
+        td = taskdata.TaskData(cooker.configuration.abort)
 
         try:
             tasks = []
@@ -170,23 +170,21 @@ class BitBakeShellCommands:
 
             td.add_unresolved(cooker.configuration.data, cooker.status)
             
-            rq = runqueue.RunQueue()
-            rq.prepare_runqueue(cooker.configuration.data, cooker.status, td, tasks)
-            rq.execute_runqueue(cooker, cooker.configuration.data, cooker.status, td, tasks)
+            rq = runqueue.RunQueue(cooker, cooker.configuration.data, cooker.status, td, tasks)
+            rq.prepare_runqueue()
+            rq.execute_runqueue()
 
         except Providers.NoProvider:
             print "ERROR: No Provider"
-            global last_exception
             last_exception = Providers.NoProvider
 
-        except runqueue.TaskFailure, (fnid, fn, taskname):
-            print "ERROR: '%s, %s' failed" % (fn, taskname)
-            global last_exception
+        except runqueue.TaskFailure, fnids:
+            for fnid in fnids:
+                print "ERROR: '%s' failed" % td.fn_index[fnid]
             last_exception = runqueue.TaskFailure
 
         except build.EventException, e:
             print "ERROR: Couldn't build '%s'" % names
-            global last_exception
             last_exception = e
 
         cooker.configuration.cmd = oldcmd
@@ -235,14 +233,13 @@ class BitBakeShellCommands:
 
     def fileBuild( self, params, cmd = "build" ):
         """Parse and build a .bb file"""
+        global last_exception
         name = params[0]
         bf = completeFilePath( name )
         print "SHELL: Calling '%s' on '%s'" % ( cmd, bf )
 
         oldcmd = cooker.configuration.cmd
         cooker.configuration.cmd = cmd
-        cooker.build_cache = []
-        cooker.build_cache_fail = []
 
         thisdata = copy.deepcopy( initdata )
         # Caution: parse.handle modifies thisdata, hence it would
@@ -254,13 +251,17 @@ class BitBakeShellCommands:
         except parse.ParseError:
             print "ERROR: Unable to open or parse '%s'" % bf
         else:
+            # Remove stamp for target if force mode active
+            if cooker.configuration.force:
+                bb.msg.note(2, bb.msg.domain.RunQueue, "Remove stamp %s, %s" % (cmd, bf))
+                bb.build.del_stamp('do_%s' % cmd, bbfile_data)
+
             item = data.getVar('PN', bbfile_data, 1)
             data.setVar( "_task_cache", [], bbfile_data ) # force
             try:
                 cooker.tryBuildPackage( os.path.abspath( bf ), item, cmd, bbfile_data, True )
             except build.EventException, e:
                 print "ERROR: Couldn't build '%s'" % name
-                global last_exception
                 last_exception = e
 
         cooker.configuration.cmd = oldcmd
@@ -296,6 +297,11 @@ class BitBakeShellCommands:
             print "SHELL: Parsed"
     fileReparse.usage = "<bbfile>"
 
+    def abort( self, params ):
+        """Toggle abort task execution flag (see bitbake -k)"""
+        cooker.configuration.abort = not cooker.configuration.abort
+        print "SHELL: Abort Flag is now '%s'" % repr( cooker.configuration.abort )
+
     def force( self, params ):
         """Toggle force task execution flag (see bitbake -f)"""
         cooker.configuration.force = not cooker.configuration.force
@@ -526,8 +532,6 @@ SRC_URI = ""
     def status( self, params ):
         """<just for testing>"""
         print "-" * 78
-        print "build cache = '%s'" % cooker.build_cache
-        print "build cache fail = '%s'" % cooker.build_cache_fail
         print "building list = '%s'" % cooker.building_list
         print "build path = '%s'" % cooker.build_path
         print "consider_msgs_cache = '%s'" % cooker.consider_msgs_cache
@@ -546,6 +550,7 @@ SRC_URI = ""
 
     def which( self, params ):
         """Computes the providers for a given providee"""
+        # Need to use taskData for this information
         item = params[0]
 
         self._checkParsed()
@@ -554,8 +559,7 @@ SRC_URI = ""
         if not preferred: preferred = item
 
         try:
-            lv, lf, pv, pf = Providers.findBestProvider(preferred, cooker.configuration.data, cooker.status, 
-cooker.build_cache_fail)
+            lv, lf, pv, pf = Providers.findBestProvider(preferred, cooker.configuration.data, cooker.status)
         except KeyError:
             lv, lf, pv, pf = (None,)*4
 
