@@ -6,19 +6,23 @@ BitBake 'TaskData' implementation
 
 Task data collection and handling
 
-Copyright (C) 2006  Richard Purdie
-
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU General Public License version 2 as published by the Free 
-Software Foundation
-
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License along with
 """
 
+# Copyright (C) 2006  Richard Purdie
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as
+# published by the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+
 from bb import data, fetch, event, mkdirhier, utils
 import bb, os
 
@@ -26,7 +30,7 @@ class TaskData:
     """
     BitBake Task Data implementation
     """
-    def __init__(self):
+    def __init__(self, abort = True):
         self.build_names_index = []
         self.run_names_index = []
         self.fn_index = []
@@ -39,6 +43,7 @@ class TaskData:
         self.tasks_fnid = []
         self.tasks_name = []
         self.tasks_tdepends = []
+        self.tasks_idepends = []
         # Cache to speed up task ID lookups
         self.tasks_lookup = {}
 
@@ -51,31 +56,7 @@ class TaskData:
         self.failed_rdeps = []
         self.failed_fnids = []
 
-
-    def matches_in_list(self, data, substring):
-        """
-        Return a list of the positions of substring in list data
-        """
-        matches = []
-        start = 0
-        while 1:
-            try:
-                start = data.index(substring, start)
-            except ValueError:
-                return matches
-            matches.append(start)
-            start = start + 1
-
-    def both_contain(self, list1, list2):
-        """
-        Return the items present in both list1 and list2
-        """
-        matches = []
-        for data in list1:
-            if data in list2:
-                return data
-        return None
-
+        self.abort = abort
 
     def getbuild_id(self, name):
         """
@@ -84,6 +65,7 @@ class TaskData:
         """
         if not name in self.build_names_index:
             self.build_names_index.append(name)
+            return len(self.build_names_index) - 1
 
         return self.build_names_index.index(name)
 
@@ -94,6 +76,7 @@ class TaskData:
         """
         if not name in self.run_names_index:
             self.run_names_index.append(name)
+            return len(self.run_names_index) - 1
 
         return self.run_names_index.index(name)
 
@@ -104,13 +87,15 @@ class TaskData:
         """
         if not name in self.fn_index:
             self.fn_index.append(name)
+            return len(self.fn_index) - 1
 
         return self.fn_index.index(name)
 
-    def gettask_id(self, fn, task):
+    def gettask_id(self, fn, task, create = True):
         """
         Return an ID number for the task matching fn and task.
-        If it doesn't exist, create one.
+        If it doesn't exist, create one by default.
+        Optionally return None instead.
         """
         fnid = self.getfn_id(fn)
 
@@ -118,9 +103,13 @@ class TaskData:
             if task in self.tasks_lookup[fnid]:
                 return self.tasks_lookup[fnid][task]
 
+        if not create:
+            return None
+
         self.tasks_name.append(task)
         self.tasks_fnid.append(fnid)
         self.tasks_tdepends.append([])
+        self.tasks_idepends.append([])
 
         listid = len(self.tasks_name) - 1
 
@@ -147,8 +136,9 @@ class TaskData:
         if fnid in self.tasks_fnid:
             return
 
-        # Work out task dependencies
         for task in task_graph.allnodes():
+
+            # Work out task dependencies
             parentids = []
             for dep in task_graph.getparents(task):
                 parentid = self.gettask_id(fn, dep)
@@ -156,28 +146,45 @@ class TaskData:
             taskid = self.gettask_id(fn, task)
             self.tasks_tdepends[taskid].extend(parentids)
 
+            # Touch all intertask dependencies
+            if 'depends' in task_deps and task in task_deps['depends']:
+                ids = []
+                for dep in task_deps['depends'][task].split(" "):
+                    if dep:
+                        ids.append(str(self.getbuild_id(dep.split(":")[0])) + ":" + dep.split(":")[1])
+                self.tasks_idepends[taskid].extend(ids)
+
         # Work out build dependencies
         if not fnid in self.depids:
-            dependids = []
+            dependids = {}
             for depend in dataCache.deps[fn]:
                 bb.msg.debug(2, bb.msg.domain.TaskData, "Added dependency %s for %s" % (depend, fn))
-                dependids.append(self.getbuild_id(depend))
-            self.depids[fnid] = dependids
+                dependids[self.getbuild_id(depend)] = None
+            self.depids[fnid] = dependids.keys()
 
         # Work out runtime dependencies
         if not fnid in self.rdepids:
-            rdependids = []
+            rdependids = {}
             rdepends = dataCache.rundeps[fn]
             rrecs = dataCache.runrecs[fn]
             for package in rdepends:
                 for rdepend in rdepends[package]:
                     bb.msg.debug(2, bb.msg.domain.TaskData, "Added runtime dependency %s for %s" % (rdepend, fn))
-                    rdependids.append(self.getrun_id(rdepend))
+                    rdependids[self.getrun_id(rdepend)] = None
             for package in rrecs:
                 for rdepend in rrecs[package]:
                     bb.msg.debug(2, bb.msg.domain.TaskData, "Added runtime recommendation %s for %s" % (rdepend, fn))
-                    rdependids.append(self.getrun_id(rdepend))
-            self.rdepids[fnid] = rdependids
+                    rdependids[self.getrun_id(rdepend)] = None
+            self.rdepids[fnid] = rdependids.keys()
+
+        for dep in self.depids[fnid]:
+            if dep in self.failed_deps:
+                self.fail_fnid(fnid)
+                return
+        for dep in self.rdepids[fnid]:
+            if dep in self.failed_rdeps:
+                self.fail_fnid(fnid)
+                return
 
     def have_build_target(self, target):
         """
@@ -318,7 +325,19 @@ class TaskData:
                 dependees.append(self.fn_index[fnid])
         return dependees
 
-    def add_provider(self, cfgData, dataCache, item, external = True):
+    def add_provider(self, cfgData, dataCache, item):
+        try:
+            self.add_provider_internal(cfgData, dataCache, item)
+        except bb.providers.NoProvider:
+            if self.abort:
+                bb.msg.error(bb.msg.domain.Provider, "No providers of build target %s (for %s)" % (item, self.get_dependees_str(item)))
+                raise
+            targetid = self.getbuild_id(item)
+            self.remove_buildtarget(targetid)
+
+        self.mark_external_target(item)
+
+    def add_provider_internal(self, cfgData, dataCache, item):
         """
         Add the providers of item to the task data
         Mark entries were specifically added externally as against dependencies 
@@ -326,19 +345,15 @@ class TaskData:
         """
 
         if item in dataCache.ignored_dependencies:
-            return True
+            return
 
         if not item in dataCache.providers:
-            msg = "No providers of build target %s (for %s)" % (item, self.get_dependees_str(item))
-            if external:
-                bb.msg.error(bb.msg.domain.Provider, msg)
-            else:
-                bb.msg.debug(1, bb.msg.domain.Provider, msg)
+            bb.msg.debug(1, bb.msg.domain.Provider, "No providers of build target %s (for %s)" % (item, self.get_dependees_str(item)))
             bb.event.fire(bb.event.NoProvider(item, cfgData))
             raise bb.providers.NoProvider(item)
 
         if self.have_build_target(item):
-            return True
+            return
 
         all_p = dataCache.providers[item]
 
@@ -350,11 +365,7 @@ class TaskData:
                 eligible.remove(p)
 
         if not eligible:
-            msg = "No providers of build target %s after filtering (for %s)" % (item, self.get_dependees_str(item))
-            if external:
-                bb.msg.error(bb.msg.domain.Provider, msg)
-            else:
-                bb.msg.debug(1, bb.msg.domain.Provider, msg)
+            bb.msg.debug(1, bb.msg.domain.Provider, "No providers of build target %s after filtering (for %s)" % (item, self.get_dependees_str(item)))
             bb.event.fire(bb.event.NoProvider(item, cfgData))
             raise bb.providers.NoProvider(item)
 
@@ -378,7 +389,7 @@ class TaskData:
                 providers_list = []
                 for fn in eligible:
                     providers_list.append(dataCache.pkg_fn[fn])
-                bb.msg.note(1, bb.msg.domain.Provider, "multiple providers are available (%s);" % ", ".join(providers_list))
+                bb.msg.note(1, bb.msg.domain.Provider, "multiple providers are available for %s (%s);" % (item, ", ".join(providers_list)))
                 bb.msg.note(1, bb.msg.domain.Provider, "consider defining PREFERRED_PROVIDER_%s" % item)
                 bb.event.fire(bb.event.MultipleProviders(item,providers_list,cfgData))
             self.consider_msgs_cache.append(item)
@@ -388,16 +399,12 @@ class TaskData:
             if fnid in self.failed_fnids:
                 continue
             bb.msg.debug(2, bb.msg.domain.Provider, "adding %s to satisfy %s" % (fn, item))
-            self.add_tasks(fn, dataCache)
             self.add_build_target(fn, item)
+            self.add_tasks(fn, dataCache)
+
 
             #item = dataCache.pkg_fn[fn]
 
-        if external:
-            self.mark_external_target(item)
-
-        return True
-
     def add_rprovider(self, cfgData, dataCache, item):
         """
         Add the runtime providers of item to the task data
@@ -405,10 +412,10 @@ class TaskData:
         """
 
         if item in dataCache.ignored_dependencies:
-            return True
+            return
 
         if self.have_runtime_target(item):
-            return True
+            return
 
         all_p = bb.providers.getRuntimeProviders(dataCache, item)
 
@@ -447,8 +454,8 @@ class TaskData:
                 providers_list = []
                 for fn in eligible:
                     providers_list.append(dataCache.pkg_fn[fn])
-                bb.msg.note(2, bb.msg.domain.Provider, "multiple providers are available (%s);" % ", ".join(providers_list))
-                bb.msg.note(2, bb.msg.domain.Provider, "consider defining a PREFERRED_PROVIDER to match runtime %s" % item)
+                bb.msg.note(2, bb.msg.domain.Provider, "multiple providers are available for runtime %s (%s);" % (item, ", ".join(providers_list)))
+                bb.msg.note(2, bb.msg.domain.Provider, "consider defining a PREFERRED_PROVIDER entry to match runtime %s" % item)
                 bb.event.fire(bb.event.MultipleProviders(item,providers_list, cfgData, runtime=True))
             self.consider_msgs_cache.append(item)
 
@@ -457,8 +464,8 @@ class TaskData:
                 providers_list = []
                 for fn in preferred:
                     providers_list.append(dataCache.pkg_fn[fn])
-                bb.msg.note(2, bb.msg.domain.Provider, "multiple preferred providers are available (%s);" % ", ".join(providers_list))
-                bb.msg.note(2, bb.msg.domain.Provider, "consider defining only one PREFERRED_PROVIDER to match runtime %s" % item)
+                bb.msg.note(2, bb.msg.domain.Provider, "multiple preferred providers are available for runtime %s (%s);" % (item, ", ".join(providers_list)))
+                bb.msg.note(2, bb.msg.domain.Provider, "consider defining only one PREFERRED_PROVIDER entry to match runtime %s" % item)
                 bb.event.fire(bb.event.MultipleProviders(item,providers_list, cfgData, runtime=True))
             self.consider_msgs_cache.append(item)
 
@@ -468,10 +475,8 @@ class TaskData:
             if fnid in self.failed_fnids:
                 continue
             bb.msg.debug(2, bb.msg.domain.Provider, "adding %s to satisfy runtime %s" % (fn, item))
-            self.add_tasks(fn, dataCache)
             self.add_runtime_target(fn, item)
-
-        return True
+            self.add_tasks(fn, dataCache)
 
     def fail_fnid(self, fnid):
         """
@@ -503,6 +508,9 @@ class TaskData:
         dependees = self.get_dependees(targetid)
         for fnid in dependees:
             self.fail_fnid(fnid)
+        if self.abort and targetid in self.external_targets:
+            bb.msg.error(bb.msg.domain.Provider, "No buildable providers available for required build target %s" % self.build_names_index[targetid])
+            raise bb.providers.NoProvider
 
     def remove_runtarget(self, targetid):
         """
@@ -524,12 +532,11 @@ class TaskData:
             added = 0
             for target in self.get_unresolved_build_targets(dataCache):
                 try:
-                    self.add_provider(cfgData, dataCache, target, False)
+                    self.add_provider_internal(cfgData, dataCache, target)
                     added = added + 1
                 except bb.providers.NoProvider:
                     targetid = self.getbuild_id(target)
-                    if targetid in self.external_targets:
-                        # FIXME - should look at configuration.abort here and only raise if set
+                    if self.abort and targetid in self.external_targets:
                         raise
                     self.remove_buildtarget(targetid)
             for target in self.get_unresolved_run_targets(dataCache):
@@ -541,7 +548,7 @@ class TaskData:
             bb.msg.debug(1, bb.msg.domain.TaskData, "Resolved " + str(added) + " extra dependecies")
             if added == 0:
                 break
-
+        # self.dump_data()
 
     def dump_data(self):
         """
@@ -564,5 +571,8 @@ class TaskData:
                 self.fn_index[self.tasks_fnid[task]], 
                 self.tasks_name[task], 
                 self.tasks_tdepends[task]))
+        bb.msg.debug(3, bb.msg.domain.TaskData, "runtime ids (per fn):")
+        for fnid in self.rdepids:
+            bb.msg.debug(3, bb.msg.domain.TaskData, " %s %s: %s" % (fnid, self.fn_index[fnid], self.rdepids[fnid]))
 
 
